[PATCH] D32213: [Sema] Use MSVC inner class behavior on Itanium

Shoaib Meenai via Phabricator via cfe-commits cfe-commits at lists.llvm.org
Wed Apr 19 18:24:33 PDT 2017


This revision was automatically updated to reflect the committed changes.
Closed by commit rL300804: [Sema] Use MSVC inner class behavior on Itanium (authored by smeenai).

Changed prior to commit:
  https://reviews.llvm.org/D32213?vs=95689&id=95867#toc

Repository:
  rL LLVM

https://reviews.llvm.org/D32213

Files:
  cfe/trunk/lib/Sema/SemaTemplateInstantiate.cpp
  cfe/trunk/test/CodeGenCXX/windows-itanium-dllexport.cpp


Index: cfe/trunk/test/CodeGenCXX/windows-itanium-dllexport.cpp
===================================================================
--- cfe/trunk/test/CodeGenCXX/windows-itanium-dllexport.cpp
+++ cfe/trunk/test/CodeGenCXX/windows-itanium-dllexport.cpp
@@ -1,5 +1,10 @@
 // RUN: %clang_cc1 -emit-llvm -triple i686-windows-itanium -fdeclspec %s -o - | FileCheck %s
 
+#define JOIN2(x, y) x##y
+#define JOIN(x, y) JOIN2(x, y)
+#define UNIQ(name) JOIN(name, __LINE__)
+#define USEMEMFUNC(class, func) void (class::*UNIQ(use)())() { return &class::func; }
+
 struct __declspec(dllexport) s {
   void f() {}
 };
@@ -28,3 +33,23 @@
 
 // CHECK: define {{.*}} dllexport {{.*}} @_ZN1cIdEaSERKS0_
 // CHECK: define {{.*}} dllexport {{.*}} @_ZN1cIdE1fEv
+
+template <class T>
+struct outer {
+  void f() {}
+  struct inner {
+    void f() {}
+  };
+};
+
+template class __declspec(dllexport) outer<int>;
+
+// CHECK: define {{.*}} dllexport {{.*}} @_ZN5outerIiE1fEv
+// CHECK-NOT: define {{.*}} dllexport {{.*}} @_ZN5outerIiE5inner1fEv
+
+extern template class __declspec(dllimport) outer<char>;
+USEMEMFUNC(outer<char>, f)
+USEMEMFUNC(outer<char>::inner, f)
+
+// CHECK: declare dllimport {{.*}} @_ZN5outerIcE1fEv
+// CHECK: define {{.*}} @_ZN5outerIcE5inner1fEv


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D32213.95867.patch
Type: text/x-patch
Size: 2154 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/cfe-commits/attachments/20170420/be512e0a/attachment.bin>


More information about the cfe-commits mailing list
Index: cfe/trunk/lib/Sema/SemaTemplateInstantiate.cpp
===================================================================
--- cfe/trunk/lib/Sema/SemaTemplateInstantiate.cpp
+++ cfe/trunk/lib/Sema/SemaTemplateInstantiate.cpp
@@ -2605,10 +2605,11 @@
                                                 == TSK_ExplicitSpecialization)
         continue;
 
-      if (Context.getTargetInfo().getCXXABI().isMicrosoft() &&
+      if ((Context.getTargetInfo().getCXXABI().isMicrosoft() ||
+           Context.getTargetInfo().getTriple().isWindowsItaniumEnvironment()) &&
           TSK == TSK_ExplicitInstantiationDeclaration) {
         // In MSVC mode, explicit instantiation decl of the outer class doesn't
         // affect the inner class.
         continue;
       }
 
@@ -2605,10 +2605,11 @@
           TSK == TSK_ExplicitInstantiationDeclaration) {
-        // In MSVC mode, explicit instantiation decl of the outer class doesn't
-        // affect the inner class.
+        // In MSVC and Windows Itanium mode, explicit instantiation decl of the
+        // outer class doesn't affect the inner class.
         continue;
       }